﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.07.13.
 * Idő: 12:34
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;

namespace TetheredSun
{
	/// <summary>
	/// Description of DataPoint.
	/// </summary>
	public class DataPoint : Point<double>
	{
		protected double error;
		protected double meanSquare;
		protected int dataCount;
		
		
		public DataPoint(double x, double y, double error) : base(x, y)
		{
			this.error = error;
			this.dataCount = -1;
			this.meanSquare = y * y;
		}
		
		public DataPoint(double x, double y) : this(x, y, Double.NaN)
		{ 
			this.dataCount = 1;
		}
		
		public int DataCount {
			get { return dataCount; }
		}
		
		public double Error {
			get { return error; }
		}
		
		
		public void Accumulate(double newY)
		{
			double sd;
			
			if (dataCount == -1) dataCount = 1;
		
			y = (dataCount * y + newY) / (dataCount + 1);
			meanSquare = (dataCount * meanSquare + newY * newY) / (dataCount + 1);
			
			dataCount++;
			
			if (dataCount < 2) {
				error = Double.NaN;
			} else {
				sd = Math.Sqrt(meanSquare - y * y);
				error = sd / Math.Sqrt(dataCount - 1);
			}
		}
		
		public void Accumulate(Point<double> point)
		{
			if (point.X != x) {
				Exception exception = new Exception("X coordinate mismatch.");
				
				exception.Data.Add("Expected x coordinate", x);
				exception.Data.Add("Received x coordinate", point.X);
				
				throw exception;
			}
			
			Accumulate(point.Y);
		}
		
		public override bool Equals(Point<double> other)
		{
			return base.Equals(other) && CompareError(other as DataPoint);
		}
		
		public override int GetHashCode()
		{
			return base.GetHashCode() ^ error.GetHashCode();
		}
		
		protected bool CompareError(DataPoint other)
		{
			if (Object.ReferenceEquals(other, null)) return false;
			if (Double.IsNaN(error)) return Double.IsNaN(other.error);
			return this.error == other.error;
		}
	}
}
