﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.07.05.
 * Idő: 11:00
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;
using System.IO;
using System.Xml;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of ComplexExtensions.
	/// </summary>
	public static class ComplexExtensions
	{
		public static void AppendToXml(this Complex complex, XmlElement element, XmlDocument context)
		{
			XmlElement child = context.CreateElement("RealPart");
			
			child.InnerXml = complex.Real.ToString();
			element.AppendChild(child);
			
			child = context.CreateElement("ImaginaryPart");
			child.InnerXml = complex.Imaginary.ToString();
			element.AppendChild(child);
		}
		
		public static Complex ParseXml(XmlElement element)
		{
			bool success = (element != null);
			double real = Double.NaN;
			double imaginary = Double.NaN;
			XmlNode node = null;
			
			if (success) {
				node = element.SelectSingleNode("child::RealPart");
				if (node == null) success = false;
			}
			
			if (success) {
				success = RobustParser.TryParse(node.InnerXml, out real);
				real = success ? real : Double.NaN;
			}
			
			if (success) {
				node = element.SelectSingleNode("child::ImaginaryPart");
				if (node == null) success = false;
			}
			
			if (success) {
				success = RobustParser.TryParse(node.InnerXml, out imaginary);
				imaginary = success ? imaginary : Double.NaN;
			}
			
			if (!success) {
				real = Double.NaN;
				imaginary = Double.NaN;
			}
			
			return new Complex(real, imaginary);
		}
	
		public static Complex ReadFromBinaryFile(BinaryReader reader)
		{
			double real;
			double imaginary;
			
			real = reader.ReadDouble();
			imaginary = reader.ReadDouble();
			
			return new Complex(real, imaginary);
		}
		
		public static void WriteToBinaryFile(this Complex complex, BinaryWriter writer)
		{
			writer.Write(complex.Real);
			writer.Write(complex.Imaginary);
		}
	}
}
