﻿/*
 * Készítette a SharpDevelop.
 * Felhasználó: phil
 * Dátum: 2011.06.08.
 * Idő: 9:53
 * 
 * A sablon megváltoztatásához használja az Eszközök | Beállítások | Kódolás | Szabvány Fejlécek Szerkesztését.
 */
using System;

namespace TetheredSun.Mathematics
{
	/// <summary>
	/// Description of Vector.
	/// </summary>
	public class Vector
	{
		protected double[] items;
		
		public Vector(int length)
		{
			this.items = new double[length];
		}
		
		public Vector(int length, double value) : this(length)
		{
			for (int i = 0; i < length; i++) {
				items[i] = value;
			}
		}
		
		protected Vector(double[] items, bool clone)
		{
			if (!clone) {
				this.items = items;
			} else {
				this.items = new double[items.Length];
				for (int i = 0; i < items.Length; i++) {
					this.items[i] = items[i];
				}
			}
		}
		
		public double this[int index] {
			get { return items[index]; }
			set { items[index] = value; }
		}
		
		public int Length {
			get { return items.Length; }
		}
		
		
		public static double[] Add(double[] vector1, double[] vector2)
		{
			int length = Math.Min(vector1.Length, vector2.Length);	
			double[] result = new double[length];
			
			for (int i = 0; i < length; i++) {
				result[i] = vector1[i] + vector2[i];
			}
			
			return result;
		}
		
		public static Vector Add(Vector vector1, Vector vector2)
		{ 
			return new Vector(Add(vector1.items, vector2.items), false);
		}
		
		public static Vector Add(double[] vector1, Vector vector2)
		{ 
			return new Vector(Add(vector1, vector2.items), false);
		}
		
		public static Vector Add(Vector vector1, double[] vector2)
		{ 
			return new Vector(Add(vector1.items, vector2), false);
		}
		
		public Vector Clone()
		{
			return new Vector(this.items, true);
		}
		
		public static double GetDotProduct(double[] vector1, double[] vector2)
		{
			if (vector1.Length != vector2.Length) {
				Exception exception = new InvalidOperationException("The lengths of the two vectors do not match.");
				
				exception.Data.Add("Length 1", vector1.Length);
				exception.Data.Add("Length 2", vector2.Length);
				
				throw exception;
			}
			
			int n = vector1.Length;
			double sum = 0.0;
			
			for (int i = 0; i < n; i++) {
				sum += vector1[i] * vector2[i];
			}
			
			return sum;
		}
		
		public static double GetDotProduct(Vector vector1, Vector vector2)
		{
			return GetDotProduct(vector1.items, vector2.items);
		}
		
		public static double GetDotProduct(double[] vector1, Vector vector2)
		{
			return GetDotProduct(vector1, vector2.items);
		}
		
		public static double GetDotProduct(Vector vector1, double[] vector2)
		{
			return GetDotProduct(vector1.items, vector2);
		}
		
		public static double[] Subtract(double[] vector1, double[] vector2)
		{
			int length = Math.Min(vector1.Length, vector2.Length);	
			double[] result = new double[length];
			
			for (int i = 0; i < length; i++) {
				result[i] = vector1[i] - vector2[i];
			}
			
			return result;
		}
		
		public static Vector Subtract(Vector vector1, Vector vector2)
		{ 
			return new Vector(Subtract(vector1.items, vector2.items), false);
		}
		
		public static Vector Subtract(double[] vector1, Vector vector2)
		{ 
			return new Vector(Subtract(vector1, vector2.items), false);
		}
		
		public static Vector Subtract(Vector vector1, double[] vector2)
		{ 
			return new Vector(Subtract(vector1.items, vector2), false);
		}
		
		public static implicit operator Vector(double[] array)
		{
			return new Vector(array, false);
		}
		
		public static implicit operator double[](Vector vector)
		{
			return vector.items;
		}
		
		public static Vector operator +(Vector vector1, Vector vector2)
		{
			return new Vector(Add(vector1.items, vector2.items), false);
		}
		
		public static Vector operator +(double[] vector1, Vector vector2)
		{
			return new Vector(Add(vector1, vector2.items), false);
		}
		
		public static Vector operator +(Vector vector1, double[] vector2)
		{
			return new Vector(Add(vector1.items, vector2), false);
		}
		
		public static Vector operator -(Vector vector1, Vector vector2)
		{
			return new Vector(Subtract(vector1.items, vector2.items), false);
		}
		
		public static Vector operator -(double[] vector1, Vector vector2)
		{
			return new Vector(Subtract(vector1, vector2.items), false);
		}
		
		public static Vector operator -(Vector vector1, double[] vector2)
		{
			return new Vector(Subtract(vector1.items, vector2), false);
		}
	}
}
