﻿/*
 * Created by SharpDevelop.
 * User: phil
 * Date: 2010.08.17.
 * Time: 13:12
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;

namespace TetheredSun.Stochastic
{
	/// <summary>
	/// Description of ExponentialRandomGenerator.
	/// </summary>
	public class ExponentialRandomGenerator : IRandomGenerator
	{
		protected double lambda = 1.0;
		protected Random uniform;
		protected double minusMean;
		protected double limit = 1E-10;
		
		public ExponentialRandomGenerator(double lambda, int seed)
		{
			this.Lambda = lambda;
			if (seed < 0) {
				uniform = new Random();
			} else {
				uniform = new Random(seed);
			}
		}
		
		public ExponentialRandomGenerator(double lambda) : this(lambda, -1) { }
		
		public double Lambda {
			get { return lambda; }
			protected set {
				if (value <= 0) {
					Exception exception = new ArgumentException("The lambda parameter must be positive.");
					exception.Data.Add("Lambda", value);
					throw exception;
				}
				lambda = value;
				minusMean = - 1.0 / lambda;
			}
		}
		
		public double ExpectedValue {
			get { return -minusMean; }
		}
		
		
		
		public double Next()
		{
			double u = uniform.NextDouble();
			
			while (1.0 - u < limit) u = uniform.NextDouble();
			
			return minusMean * Math.Log(1.0 - u);
		}
		
		public double[] GenerateSequence(int count)
		{
			double[] sequence = new double[count];
			
			for (int i = 0; i < count; i++) {
				sequence[i] = Next();
			}
			
			return sequence;
		}
		
		public double ProbabilityDensity(double x)
		{
			return (x < 0) ? 0.0 : lambda * Math.Exp(-lambda * x);
		}
		
		
	}
}
